@@ -1,3 +1,5 @@
+from __future__ import annotations
+
 import functools
 import logging
 import os
@@ -5,9 +7,10 @@ import shutil
 import sys
 import uuid
 import zipfile
+from collections.abc import Collection, Iterable, Sequence
 from optparse import Values
 from pathlib import Path
-from typing import Any, Collection, Dict, Iterable, List, Optional, Sequence, Union
+from typing import Any
 
 from pip._vendor.packaging.markers import Marker
 from pip._vendor.packaging.requirements import Requirement
@@ -52,7 +55,7 @@ from pip._internal.utils.misc import (
     redact_auth_from_requirement,
     redact_auth_from_url,
 )
-from pip._internal.utils.packaging import safe_extra
+from pip._internal.utils.packaging import get_requirement
 from pip._internal.utils.subprocess import runner_with_spinner_message
 from pip._internal.utils.temp_dir import TempDirectory, tempdir_kinds
 from pip._internal.utils.unpacking import unpack_file
@@ -71,17 +74,17 @@ class InstallRequirement:
 
     def __init__(
         self,
-        req: Optional[Requirement],
-        comes_from: Optional[Union[str, "InstallRequirement"]],
+        req: Requirement | None,
+        comes_from: str | InstallRequirement | None,
         editable: bool = False,
-        link: Optional[Link] = None,
-        markers: Optional[Marker] = None,
-        use_pep517: Optional[bool] = None,
+        link: Link | None = None,
+        markers: Marker | None = None,
+        use_pep517: bool | None = None,
         isolated: bool = False,
         *,
-        global_options: Optional[List[str]] = None,
-        hash_options: Optional[Dict[str, List[str]]] = None,
-        config_settings: Optional[Dict[str, Union[str, List[str]]]] = None,
+        global_options: list[str] | None = None,
+        hash_options: dict[str, list[str]] | None = None,
+        config_settings: dict[str, str | list[str]] | None = None,
         constraint: bool = False,
         extras: Collection[str] = (),
         user_supplied: bool = False,
@@ -99,7 +102,7 @@ class InstallRequirement:
         # populating source_dir is done by the RequirementPreparer. Note this
         # is not necessarily the directory where pyproject.toml or setup.py is
         # located - that one is obtained via unpacked_source_directory.
-        self.source_dir: Optional[str] = None
+        self.source_dir: str | None = None
         if self.editable:
             assert link
             if link.is_file:
@@ -115,14 +118,14 @@ class InstallRequirement:
         # When this InstallRequirement is a wheel obtained from the cache of locally
         # built wheels, this is the source link corresponding to the cache entry, which
         # was used to download and build the cached wheel.
-        self.cached_wheel_source_link: Optional[Link] = None
+        self.cached_wheel_source_link: Link | None = None
 
         # Information about the location of the artifact that was downloaded . This
         # property is guaranteed to be set in resolver results.
-        self.download_info: Optional[DirectUrl] = None
+        self.download_info: DirectUrl | None = None
 
         # Path to any downloaded or already-existing package.
-        self.local_file_path: Optional[str] = None
+        self.local_file_path: str | None = None
         if self.link and self.link.is_file:
             self.local_file_path = self.link.file_path
 
@@ -137,14 +140,14 @@ class InstallRequirement:
         self.markers = markers
 
         # This holds the Distribution object if this requirement is already installed.
-        self.satisfied_by: Optional[BaseDistribution] = None
+        self.satisfied_by: BaseDistribution | None = None
         # Whether the installation process should try to uninstall an existing
         # distribution before installing this requirement.
         self.should_reinstall = False
         # Temporary build location
-        self._temp_build_dir: Optional[TempDirectory] = None
+        self._temp_build_dir: TempDirectory | None = None
         # Set to True after successful installation
-        self.install_succeeded: Optional[bool] = None
+        self.install_succeeded: bool | None = None
         # Supplied options
         self.global_options = global_options if global_options else []
         self.hash_options = hash_options if hash_options else {}
@@ -163,16 +166,16 @@ class InstallRequirement:
         # gets stored. We need this to pass to build_wheel, so the backend
         # can ensure that the wheel matches the metadata (see the PEP for
         # details).
-        self.metadata_directory: Optional[str] = None
+        self.metadata_directory: str | None = None
 
         # The static build requirements (from pyproject.toml)
-        self.pyproject_requires: Optional[List[str]] = None
+        self.pyproject_requires: list[str] | None = None
 
         # Build requirements that we will check are available
-        self.requirements_to_check: List[str] = []
+        self.requirements_to_check: list[str] = []
 
         # The PEP 517 backend we should use to build the project
-        self.pep517_backend: Optional[BuildBackendHookCaller] = None
+        self.pep517_backend: BuildBackendHookCaller | None = None
 
         # Are we using PEP 517 for this requirement?
         # After pyproject.toml has been loaded, the only valid values are True
@@ -181,17 +184,27 @@ class InstallRequirement:
         # but after loading this flag should be treated as read only.
         self.use_pep517 = use_pep517
 
+        # If config settings are provided, enforce PEP 517.
+        if self.config_settings:
+            if self.use_pep517 is False:
+                logger.warning(
+                    "--no-use-pep517 ignored for %s "
+                    "because --config-settings are specified.",
+                    self,
+                )
+            self.use_pep517 = True
+
         # This requirement needs more preparation before it can be built
         self.needs_more_preparation = False
 
         # This requirement needs to be unpacked before it can be installed.
-        self._archive_source: Optional[Path] = None
+        self._archive_source: Path | None = None
 
     def __str__(self) -> str:
         if self.req:
             s = redact_auth_from_requirement(self.req)
             if self.link:
-                s += " from {}".format(redact_auth_from_url(self.link.url))
+                s += f" from {redact_auth_from_url(self.link.url)}"
         elif self.link:
             s = redact_auth_from_url(self.link.url)
         else:
@@ -204,7 +217,7 @@ class InstallRequirement:
             s += f" in {location}"
         if self.comes_from:
             if isinstance(self.comes_from, str):
-                comes_from: Optional[str] = self.comes_from
+                comes_from: str | None = self.comes_from
             else:
                 comes_from = self.comes_from.from_path()
             if comes_from:
@@ -212,8 +225,9 @@ class InstallRequirement:
         return s
 
     def __repr__(self) -> str:
-        return "<{} object: {} editable={!r}>".format(
-            self.__class__.__name__, str(self), self.editable
+        return (
+            f"<{self.__class__.__name__} object: "
+            f"{str(self)} editable={self.editable!r}>"
         )
 
     def format_debug(self) -> str:
@@ -221,7 +235,7 @@ class InstallRequirement:
         attributes = vars(self)
         names = sorted(attributes)
 
-        state = ("{}={!r}".format(attr, attributes[attr]) for attr in sorted(names))
+        state = (f"{attr}={attributes[attr]!r}" for attr in sorted(names))
         return "<{name} object: {{{state}}}>".format(
             name=self.__class__.__name__,
             state=", ".join(state),
@@ -229,12 +243,12 @@ class InstallRequirement:
 
     # Things that are valid for all kinds of requirements?
     @property
-    def name(self) -> Optional[str]:
+    def name(self) -> str | None:
         if self.req is None:
             return None
         return self.req.name
 
-    @functools.lru_cache()  # use cached_property in python 3.8+
+    @functools.cached_property
     def supports_pyproject_editable(self) -> bool:
         if not self.use_pep517:
             return False
@@ -266,19 +280,14 @@ class InstallRequirement:
         specifiers = self.req.specifier
         return len(specifiers) == 1 and next(iter(specifiers)).operator in {"==", "==="}
 
-    def match_markers(self, extras_requested: Optional[Iterable[str]] = None) -> bool:
+    def match_markers(self, extras_requested: Iterable[str] | None = None) -> bool:
         if not extras_requested:
             # Provide an extra to safely evaluate the markers
             # without matching any extra
             extras_requested = ("",)
         if self.markers is not None:
             return any(
-                self.markers.evaluate({"extra": extra})
-                # TODO: Remove these two variants when packaging is upgraded to
-                # support the marker comparison logic specified in PEP 685.
-                or self.markers.evaluate({"extra": safe_extra(extra)})
-                or self.markers.evaluate({"extra": canonicalize_name(extra)})
-                for extra in extras_requested
+                self.markers.evaluate({"extra": extra}) for extra in extras_requested
             )
         else:
             return True
@@ -320,13 +329,13 @@ class InstallRequirement:
             good_hashes.setdefault(link.hash_name, []).append(link.hash)
         return Hashes(good_hashes)
 
-    def from_path(self) -> Optional[str]:
+    def from_path(self) -> str | None:
         """Format a nice indicator to show where this "comes from" """
         if self.req is None:
             return None
         s = str(self.req)
         if self.comes_from:
-            comes_from: Optional[str]
+            comes_from: str | None
             if isinstance(self.comes_from, str):
                 comes_from = self.comes_from
             else:
@@ -390,7 +399,7 @@ class InstallRequirement:
         else:
             op = "==="
 
-        self.req = Requirement(
+        self.req = get_requirement(
             "".join(
                 [
                     self.metadata["Name"],
@@ -416,7 +425,7 @@ class InstallRequirement:
             metadata_name,
             self.name,
         )
-        self.req = Requirement(metadata_name)
+        self.req = get_requirement(metadata_name)
 
     def check_if_exists(self, use_user_site: bool) -> None:
         """Find an installed distribution that satisfies or conflicts
@@ -508,15 +517,7 @@ class InstallRequirement:
         )
 
         if pyproject_toml_data is None:
-            if self.config_settings:
-                deprecated(
-                    reason=f"Config settings are ignored for project {self}.",
-                    replacement=(
-                        "to use --use-pep517 or add a "
-                        "pyproject.toml file to the project"
-                    ),
-                    gone_in="24.0",
-                )
+            assert not self.config_settings
             self.use_pep517 = False
             return
 
@@ -540,7 +541,7 @@ class InstallRequirement:
         if (
             self.editable
             and self.use_pep517
-            and not self.supports_pyproject_editable()
+            and not self.supports_pyproject_editable
             and not os.path.isfile(self.setup_py_path)
             and not os.path.isfile(self.setup_cfg_path)
         ):
@@ -566,7 +567,7 @@ class InstallRequirement:
             if (
                 self.editable
                 and self.permit_editable_wheels
-                and self.supports_pyproject_editable()
+                and self.supports_pyproject_editable
             ):
                 self.metadata_directory = generate_editable_metadata(
                     build_env=self.build_env,
@@ -701,7 +702,7 @@ class InstallRequirement:
     # Top-level Actions
     def uninstall(
         self, auto_confirm: bool = False, verbose: bool = False
-    ) -> Optional[UninstallPathSet]:
+    ) -> UninstallPathSet | None:
         """
         Uninstall the distribution currently satisfying this requirement.
 
@@ -739,7 +740,7 @@ class InstallRequirement:
         name = _clean_zip_name(path, rootdir)
         return self.req.name + "/" + name
 
-    def archive(self, build_dir: Optional[str]) -> None:
+    def archive(self, build_dir: str | None) -> None:
         """Saves archive to provided build_dir.
 
         Used for saving downloaded VCS requirements as part of `pip download`.
@@ -754,8 +755,8 @@ class InstallRequirement:
 
         if os.path.exists(archive_path):
             response = ask_path_exists(
-                "The file {} exists. (i)gnore, (w)ipe, "
-                "(b)ackup, (a)bort ".format(display_path(archive_path)),
+                f"The file {display_path(archive_path)} exists. (i)gnore, (w)ipe, "
+                "(b)ackup, (a)bort ",
                 ("i", "w", "b", "a"),
             )
             if response == "i":
@@ -808,11 +809,11 @@ class InstallRequirement:
 
     def install(
         self,
-        global_options: Optional[Sequence[str]] = None,
-        root: Optional[str] = None,
-        home: Optional[str] = None,
-        prefix: Optional[str] = None,
-        executable: Optional[str] = None,
+        global_options: Sequence[str] | None = None,
+        root: str | None = None,
+        home: str | None = None,
+        prefix: str | None = None,
+        executable: str | None = None,
         warn_script_location: bool = True,
         use_user_site: bool = False,
         pycompile: bool = True,
@@ -828,6 +829,28 @@ class InstallRequirement:
         )
 
         if self.editable and not self.is_wheel:
+            deprecated(
+                reason=(
+                    f"Legacy editable install of {self} (setup.py develop) "
+                    "is deprecated."
+                ),
+                replacement=(
+                    "to add a pyproject.toml or enable --use-pep517, "
+                    "and use setuptools >= 64. "
+                    "If the resulting installation is not behaving as expected, "
+                    "try using --config-settings editable_mode=compat. "
+                    "Please consult the setuptools documentation for more information"
+                ),
+                gone_in="25.3",
+                issue=11457,
+            )
+            if self.config_settings:
+                logger.warning(
+                    "--config-settings ignored for legacy editable install of %s. "
+                    "Consider upgrading to a version of setuptools "
+                    "that supports PEP 660 (>= 64).",
+                    self,
+                )
             install_editable_legacy(
                 global_options=global_options if global_options is not None else [],
                 prefix=prefix,
@@ -887,7 +910,7 @@ def check_invalid_constraint_type(req: InstallRequirement) -> str:
     return problem
 
 
-def _has_option(options: Values, reqs: List[InstallRequirement], option: str) -> bool:
+def _has_option(options: Values, reqs: list[InstallRequirement], option: str) -> bool:
     if getattr(options, option, None):
         return True
     for req in reqs:
@@ -898,7 +921,7 @@ def _has_option(options: Values, reqs: List[InstallRequirement], option: str) ->
 
 def check_legacy_setup_py_options(
     options: Values,
-    reqs: List[InstallRequirement],
+    reqs: list[InstallRequirement],
 ) -> None:
     has_build_options = _has_option(options, reqs, "build_options")
     has_global_options = _has_option(options, reqs, "global_options")
@@ -907,7 +930,7 @@ def check_legacy_setup_py_options(
             reason="--build-option and --global-option are deprecated.",
             issue=11859,
             replacement="to use --config-settings",
-            gone_in="24.0",
+            gone_in="25.3",
         )
         logger.warning(
             "Implying --no-binary=:all: due to the presence of "
